/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include "typeconv.hpp"

namespace nostalgia::core {

ox::Error NostalgiaPaletteToPaletteV1Converter::convert(
		keel::Context&,
		NostalgiaPalette &src,
		PaletteV1 &dst) const noexcept {
	dst.colors = std::move(src.colors);
	return {};
}

ox::Error PaletteV1ToPaletteV2Converter::convert(
		keel::Context&,
		PaletteV1 &src,
		PaletteV2 &dst) const noexcept {
	dst.pages.emplace_back(std::move(src.colors));
	return {};
}

ox::Error PaletteV2ToPaletteV3Converter::convert(
		keel::Context&,
		PaletteV2 &src,
		PaletteV3 &dst) const noexcept {
	dst.pages = std::move(src.pages);
	if (!dst.pages.empty()) {
		dst.colorInfo.reserve(dst.pages[0].size());
		for (size_t i = 0; i < dst.pages[0].size(); ++i) {
			dst.colorInfo.emplace_back(ox::sfmt("Color {}", i + 1));
		}
	}
	return {};
}

ox::Error PaletteV3ToPaletteV4Converter::convert(
		keel::Context&,
		PaletteV3 &src,
		PaletteV4 &dst) const noexcept {
	dst.pages.reserve(src.pages.size());
	for (auto i = 1; auto &page : src.pages) {
		dst.pages.emplace_back(ox::sfmt("Page {}", i), std::move(page));
		++i;
	}
	dst.colorNames.reserve(src.colorInfo.size());
	for (auto &ci : src.colorInfo) {
		dst.colorNames.emplace_back(std::move(ci.name));
	}
	return {};
}

ox::Error PaletteToCompactPaletteConverter::convert(
		keel::Context&,
		Palette &src,
		CompactPalette &dst) const noexcept {
	dst.pages.reserve(src.pages.size());
	for (auto &page : src.pages) {
		auto &p = dst.pages.emplace_back();
		for (auto const c : page.colors) {
			p.emplace_back(c);
		}
	}
	return {};
}

ox::Error TileSheetV1ToTileSheetV2Converter::convert(
		keel::Context&,
		TileSheetV1 &src,
		TileSheetV2 &dst) const noexcept {
	dst.bpp              = src.bpp;
	dst.defaultPalette   = std::move(src.defaultPalette);
	dst.subsheet.name    = "Root";
	dst.subsheet.rows    = src.rows;
	dst.subsheet.columns = src.columns;
	dst.subsheet.pixels  = std::move(src.pixels);
	return {};
}

void TileSheetV2ToTileSheetV3Converter::convertSubsheet(
		TileSheetV2::SubSheet &src,
		TileSheetV3::SubSheet &dst,
		SubSheetId &idIt) noexcept {
	dst.id      = idIt;
	dst.name    = std::move(src.name);
	dst.columns = src.columns;
	dst.rows    = src.rows;
	dst.pixels  = std::move(src.pixels);
	++idIt;
	dst.subsheets.resize(src.subsheets.size());
	for (auto i = 0u; i < src.subsheets.size(); ++i) {
		convertSubsheet(src.subsheets[i], dst.subsheets[i], idIt);
	}
}

ox::Error TileSheetV2ToTileSheetV3Converter::convert(
		keel::Context&,
		TileSheetV2 &src,
		TileSheetV3 &dst) const noexcept {
	dst.bpp            = src.bpp;
	dst.defaultPalette = std::move(src.defaultPalette);
	convertSubsheet(src.subsheet, dst.subsheet, dst.idIt);
	return {};
}


void TileSheetV3ToTileSheetV4Converter::convertSubsheet(
		TileSheetV3::SubSheet &src,
		TileSheetV4::SubSheet &dst,
		SubSheetId &idIt) noexcept {
	dst.id      = idIt;
	dst.name    = std::move(src.name);
	dst.columns = src.columns;
	dst.rows    = src.rows;
	dst.pixels  = std::move(src.pixels);
	++idIt;
	dst.subsheets.resize(src.subsheets.size());
	for (auto i = 0u; i < src.subsheets.size(); ++i) {
		convertSubsheet(src.subsheets[i], dst.subsheets[i], idIt);
	}
}

ox::Error TileSheetV3ToTileSheetV4Converter::convert(
		keel::Context&,
		TileSheetV3 &src,
		TileSheetV4 &dst) const noexcept {
	dst.bpp            = src.bpp;
	dst.idIt           = src.idIt;
	dst.defaultPalette = std::move(src.defaultPalette);
	convertSubsheet(src.subsheet, dst.subsheet, dst.idIt);
	return {};
}

ox::Error TileSheetToCompactTileSheetConverter::convert(
		keel::Context&,
		TileSheet &src,
		CompactTileSheet &dst) const noexcept {
	dst.bpp            = src.bpp;
	dst.defaultPalette = std::move(src.defaultPalette);
	dst.pixels         = pixels(src);
	return {};
}

}
