/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <studio/studio.hpp>

#include <nostalgia/core/gfx.hpp>
#include <nostalgia/core/palette.hpp>

namespace nostalgia::core {

class PaletteEditorImGui: public studio::Editor {

	private:
		class PageRename {
			private:
				ox::IString<50> m_name;
				bool m_show = false;
			public:
				ox::Signal<ox::Error(ox::StringView name)> inputSubmitted;
				constexpr void show(ox::StringView const&name) noexcept {
					m_show = true;
					m_name = name;
				}
				constexpr void close() noexcept {
					m_show = false;
				}
				[[nodiscard]]
				constexpr bool isOpen() const noexcept { return m_show; }
				void draw(turbine::Context &tctx) noexcept;
		} m_pageRename;
		studio::StudioContext &m_sctx;
		turbine::Context &m_tctx;
		Palette m_pal;
		size_t m_selectedColorRow = 0;
		size_t m_page = 0;

	public:
		PaletteEditorImGui(studio::StudioContext &sctx, ox::StringParam path);

		void draw(studio::StudioContext&) noexcept final;

	protected:
		ox::Error saveItem() noexcept final;

	private:
		static void drawColumnLeftAlign(ox::CStringView txt) noexcept;

		static void drawColumn(ox::CStringView txt) noexcept;

		static void drawColumn(ox::Integer_c auto i) noexcept {
			drawColumn(ox::itoa(i));
		}

		static void numShortcuts(size_t &val, size_t sizeRange) noexcept;

		static void colorInput(ox::CStringView label, int &v, bool &inputFocused) noexcept;

		void drawColorsEditor() noexcept;

		void drawPagesEditor() noexcept;

		void drawColorEditor() noexcept;

		ox::Error renamePage(ox::StringView name) noexcept;

		ox::Error handleCommand(studio::UndoCommand const*) noexcept;
};

}
