/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <keel/asset.hpp>

namespace keel {

ox::Result<ox::UUID> readUuidHeader(ox::BufferView buff) noexcept {
	if (buff.size() < K1HdrSz) [[unlikely]] {
		return OxError(1, "Insufficient data to contain complete Keel header");
	}
	constexpr ox::StringView k1Hdr = "K1;";
	if (k1Hdr != ox::StringView(buff.data(), k1Hdr.bytes())) [[unlikely]] {
		return OxError(2, "No Keel asset header data");
	}
	return ox::UUID::fromString(ox::StringView(buff.data() + k1Hdr.bytes(), 36));
}

ox::Result<ox::ModelObject> readAsset(ox::TypeStore &ts, ox::BufferView buff) noexcept {
	std::size_t offset = 0;
	if (!readUuidHeader(buff).error) {
		offset = K1HdrSz;
	}
	buff += offset;
	return ox::readClaw(ts, buff);
}

ox::Result<ox::StringView> readAssetTypeId(ox::BufferView buff) noexcept {
	const auto err = readUuidHeader(buff).error;
	const auto offset = err ? 0u : K1HdrSz;
	if (offset >= buff.size()) [[unlikely]] {
		return OxError(1, "Buffer too small for expected data");
	}
	return ox::readClawTypeId(buff + offset);
}

ox::Result<AssetHdr> readAssetHeader(ox::BufferView buff) noexcept {
	ox::Result<AssetHdr> out;
	const auto err = readUuidHeader(buff).moveTo(out.value.uuid);
	const auto offset = err ? 0u : K1HdrSz;
	if (offset >= buff.size()) [[unlikely]] {
		return OxError(1, "Buffer too small for expected data");
	}
	buff += offset;
	oxReturnError(ox::readClawHeader(buff).moveTo(out.value.clawHdr));
	return out;
}

}
