/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/claw/claw.hpp>

#include <keel/media.hpp>
#include <turbine/context.hpp>

#include "context.hpp"

namespace studio {

class ItemMaker {
	public:
		ox::String const typeName;
		ox::String const parentDir;
		ox::String const fileExt;
		constexpr explicit ItemMaker(
				ox::StringParam pName,
				ox::StringParam pParentDir,
				ox::StringParam pFileExt) noexcept:
			typeName{std::move(pName)},
			parentDir{std::move(pParentDir)},
			fileExt{std::move(pFileExt)} {
		}
		virtual ~ItemMaker() noexcept = default;

		[[nodiscard]]
		virtual ox::String itemPath(ox::StringView pName) const noexcept {
			return ox::sfmt("/{}/{}.{}", parentDir, pName, fileExt);
		}

		/**
		 * Returns path of the file created.
		 * @param ctx
		 * @param pName
		 * @return path of file or error in Result
		 */
		virtual ox::Result<ox::String> write(StudioContext &ctx, ox::StringView pName) const noexcept = 0;
};

template<typename T>
class ItemMakerT: public ItemMaker {
	private:
		T const m_item;
		ox::ClawFormat const m_fmt;
	public:
		constexpr ItemMakerT(
				ox::StringParam pDisplayName,
				ox::StringParam pParentDir,
				ox::StringParam fileExt,
				ox::ClawFormat pFmt = ox::ClawFormat::Metal) noexcept:
			ItemMaker(std::move(pDisplayName), std::move(pParentDir), std::move(fileExt)),
			m_fmt(pFmt) {
		}
		constexpr ItemMakerT(
				ox::StringParam pDisplayName,
				ox::StringParam pParentDir,
				ox::StringParam fileExt,
				T pItem,
				ox::ClawFormat pFmt) noexcept:
			ItemMaker(std::move(pDisplayName), std::move(pParentDir), std::move(fileExt)),
			m_item(std::move(pItem)),
			m_fmt(pFmt) {
		}
		constexpr ItemMakerT(
				ox::StringParam pDisplayName,
				ox::StringParam pParentDir,
				ox::StringParam fileExt,
				T &&pItem,
				ox::ClawFormat pFmt) noexcept:
			ItemMaker(std::move(pDisplayName), std::move(pParentDir), std::move(fileExt)),
			m_item(std::move(pItem)),
			m_fmt(pFmt) {
		}
		ox::Result<ox::String> write(studio::StudioContext &sctx, ox::StringView const pName) const noexcept override {
			auto const path = itemPath(pName);
			createUuidMapping(keelCtx(sctx.tctx), path, ox::UUID::generate().unwrap());
			oxReturnError(sctx.project->writeObj(path, m_item, m_fmt));
			return path;
		}
};

}
