/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <functional>

#include <ox/std/string.hpp>
#include <ox/std/vector.hpp>

#include <turbine/context.hpp>

#include <studio/itemmaker.hpp>

namespace studio {

class ItemMaker;

struct EditorMaker {
	using Func = std::function<ox::Result<class BaseEditor*>(ox::StringViewCR)>;
	ox::Vector<ox::String> fileTypes;
	Func make;
};

class Module {
	public:
		virtual ~Module() noexcept = default;

		virtual ox::Vector<EditorMaker> editors(studio::StudioContext &ctx) const;

		virtual ox::Vector<ox::UPtr<ItemMaker>> itemMakers(studio::StudioContext&) const;

};

template<typename Editor>
[[nodiscard]]
studio::EditorMaker editorMaker(studio::StudioContext &ctx, ox::StringParam ext) noexcept {
	return {
			{std::move(ext)},
			[&ctx](ox::StringViewCR path) -> ox::Result<studio::BaseEditor*> {
				return ox::makeCatch<Editor>(ctx, path);
			}
	};
}

}
