/*
 * Copyright 2016 - 2024 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#include <studio/undostack.hpp>

namespace studio {

ox::Error UndoStack::push(ox::UPtr<UndoCommand> &&cmd) noexcept {
	for (auto const i = m_stackIdx; i < m_stack.size();) {
		std::ignore = m_stack.erase(i);
	}
	oxReturnError(cmd->redo());
	redoTriggered.emit(cmd.get());
	changeTriggered.emit(cmd.get());
	if (m_stack.empty() || !(*m_stack.back().value)->mergeWith(*cmd)) {
		m_stack.emplace_back(std::move(cmd));
		++m_stackIdx;
	}
	if ((*m_stack.back().unwrap())->isObsolete()) {
		m_stack.pop_back();
		--m_stackIdx;
	}
	return {};
}

ox::Error UndoStack::redo() noexcept {
	if (m_stackIdx < m_stack.size()) {
		auto &c = m_stack[m_stackIdx];
		oxReturnError(c->redo());
		++m_stackIdx;
		redoTriggered.emit(c.get());
		changeTriggered.emit(c.get());
	}
	return {};
}

ox::Error UndoStack::undo() noexcept {
	if (m_stackIdx) {
		auto &c = m_stack[--m_stackIdx];
		oxReturnError(c->undo());
		undoTriggered.emit(c.get());
		changeTriggered.emit(c.get());
	}
	return {};
}

}
